@@ -1,7 +1,7 @@
 import re
 import logging
 
-from numpy.testing.utils import assert_equal, assert_raises, assert_allclose
+from numpy.testing.utils import assert_equal, assert_raises
 from nose.plugins.attrib import attr
 from nose import with_setup
 
@@ -10,6 +10,8 @@ from brian2.utils.logger import catch_logs
 from brian2.core.variables import ArrayVariable, Variable, Constant
 from brian2.devices.device import reinit_devices
 from brian2.stateupdaters.base import UnsupportedEquationsException
+from brian2.tests.utils import assert_allclose
+
 
 @attr('codegen-independent')
 def test_explicit_stateupdater_parsing():
@@ -320,6 +322,26 @@ def test_illegal_calls():
                                                                           group_name='my_name',
                                                                           method=[object(), 'euler']))
 
+def check_integration(eqs, variables, can_integrate):
+    # can_integrate maps integrators to True/False/None
+    # True/False means that the integrator should/should not integrate the equations
+    # None means that it *might* integrate the equations (only needed for the
+    # exact integration, since it can depend on the sympy version)
+    for integrator, able in can_integrate.items():
+        try:
+            integrator(eqs, variables)
+            if able is False:
+                raise AssertionError('Should not be able to integrate these '
+                                     'equations (equations: "{}") with '
+                                     'integrator {}'.format(eqs,
+                                                            integrator.__class__.__name__))
+        except UnsupportedEquationsException:
+            if able is True:
+                raise AssertionError('Should be able to integrate these '
+                                     'equations (equations: "{}") with '
+                                     'integrator {}'.format(eqs,
+                                                            integrator.__class__.__name__))
+
 @attr('codegen-independent')
 def test_priority():
     updater = ExplicitStateUpdater('x_new = x + dt * f(x, t)')
@@ -329,6 +351,9 @@ def test_priority():
     variables = {'v': ArrayVariable(name='name', size=10,
                                     owner=None, device=None, dtype=np.float64,
                                     constant=False),
+                 'w': ArrayVariable(name='name', size=10,
+                                    owner=None, device=None, dtype=np.float64,
+                                    constant=False),
                  't': clock.variables['t'],
                  'dt': clock.variables['dt']}
     updater(eqs, variables)  # should not raise an error
@@ -337,73 +362,51 @@ def test_priority():
     param = 1
     eqs = Equations('dv/dt = -param * v / (10*ms) : 1')
     updater(eqs, variables)  # should not raise an error
-    can_integrate = {linear: True, euler: True, rk2: True, rk4: True, 
-                     heun: True, milstein: True}
-    for integrator, able in can_integrate.items():
-        try:
-            integrator(eqs, variables)
-            if not able:
-                raise AssertionError('Should not be able to integrate these '
-                                     'equations')
-        except UnsupportedEquationsException:
-            if able:
-                raise AssertionError('Should be able to integrate these '
-                                     'equations')
+    can_integrate = {linear: True, euler: True, exponential_euler: True,
+                     rk2: True, rk4: True, heun: True, milstein: True}
 
-    # Equations resulting in complex linear solution (unsupported)
+    check_integration(eqs, variables, can_integrate)
+
+    # Constant equation, should work for all except linear (see #1010)
+    param = 1
+    eqs = Equations('''dv/dt = 10*Hz : 1
+                       dw/dt = -v/(10*ms) : 1''')
+    updater(eqs, variables)  # should not raise an error
+    can_integrate = {linear: None, euler: True, exponential_euler: True,
+                     rk2: True, rk4: True, heun: True, milstein: True}
+
+    check_integration(eqs, variables, can_integrate)
+
+    # Equations resulting in complex linear solution for older versions of sympy
     eqs = Equations('''dv/dt      = (ge+gi-(v+49*mV))/(20*ms) : volt
             dge/dt     = -ge/(5*ms) : volt
             dgi/dt     = Dgi/(5*ms) : volt
             dDgi/dt    = ((-2./5) * Dgi - (1./5**2)*gi)/(10*ms) : volt''')
-    can_integrate = {linear: False, euler: True, rk2: True, rk4: True,
-                     heun: True, milstein: True}
-    for integrator, able in can_integrate.items():
-        try:
-            integrator(eqs, variables)
-            if not able:
-                raise AssertionError('Should not be able to integrate these '
-                                     'equations')
-        except UnsupportedEquationsException:
-            if able:
-                raise AssertionError('Should be able to integrate these '
-                                     'equations')
+    can_integrate = {linear: None, euler: True, exponential_euler: True,
+                     rk2: True, rk4: True, heun: True, milstein: True}
+    check_integration(eqs, variables, can_integrate)
+
 
     # Equation with additive noise
     eqs = Equations('dv/dt = -v / (10*ms) + xi/(10*ms)**.5 : 1')
     assert_raises(UnsupportedEquationsException,
                   lambda: updater(eqs, variables))
     
-    can_integrate = {linear: False, euler: True, rk2: False, rk4: False, 
-                     heun: True, milstein: True}
-    for integrator, able in can_integrate.items():
-        try:
-            integrator(eqs, variables)
-            if not able:
-                raise AssertionError('Should not be able to integrate these '
-                                     'equations')
-        except UnsupportedEquationsException:
-            if able:
-                raise AssertionError('Should be able to integrate these '
-                                     'equations')
+    can_integrate = {linear: False, euler: True, exponential_euler: False,
+                     rk2: False, rk4: False, heun: True, milstein: True}
+
+    check_integration(eqs, variables, can_integrate)
     
     # Equation with multiplicative noise
     eqs = Equations('dv/dt = -v / (10*ms) + v*xi/(10*ms)**.5 : 1')
     assert_raises(UnsupportedEquationsException,
                   lambda: updater(eqs, variables))
     
-    can_integrate = {linear: False, euler: False, rk2: False, rk4: False, 
-                     heun: True, milstein: True}
-    for integrator, able in can_integrate.items():
-        try:
-            integrator(eqs, variables)
-            if not able:
-                raise AssertionError('Should not be able to integrate these '
-                                     'equations')
-        except UnsupportedEquationsException:
-            if able:
-                raise AssertionError('Should be able to integrate these '
-                                     'equations')
-    
+    can_integrate = {linear: False, euler: False, exponential_euler: False,
+                     rk2: False, rk4: False, heun: True, milstein: True}
+                     
+    check_integration(eqs, variables, can_integrate)
+
 
 @attr('codegen-independent')
 def test_registration():
@@ -722,19 +725,21 @@ def test_check_for_invalid_values_linear_integrator():
     G.x = 1
     BrianLogger._log_messages.clear() # because the log message is set to be shown only once
     with catch_logs() as clog:
-        run(1*ms)
-        # this check allows for the possibility that we improve the linear
-        # integrator in the future so that it can handle this equation
-        if numpy.isnan(G.x[0]):
-            assert 'invalid_values' in repr(clog)
-        else:
-            assert G.x[0] != 0
+        try:
+            run(1*ms)
+            # this check allows for the possibility that we improve the linear
+            # integrator in the future so that it can handle this equation
+            if numpy.isnan(G.x[0]):
+                assert 'invalid_values' in repr(clog)
+            else:
+                assert G.x[0] != 0
+        except UnsupportedEquationsException:
+            pass
 
 
 if __name__ == '__main__':
     from brian2 import prefs
-    # prefs.codegen.target = 'cython'
-    # set_device('cpp_standalone')
+
     import time
     start = time.time()
 
